@@ -19,45 +19,18 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include "pango-fontmap.h"
-#include "pango-utils.h"
+#include "pango-impl-utils.h"
 #include <stdlib.h>
 
-static void          pango_font_map_class_init        (PangoFontMapClass          *class);
 static PangoFontset *pango_font_map_real_load_fontset (PangoFontMap               *fontmap,
 						       PangoContext               *context,
 						       const PangoFontDescription *desc,
 						       PangoLanguage              *language);
 
 
-GType
-pango_font_map_get_type (void)
-{
-  static GType object_type = 0;
-
-  if (!object_type)
-    {
-      static const GTypeInfo object_info =
-      {
-        sizeof (PangoFontMapClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) pango_font_map_class_init,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (PangoFontMap),
-        0,              /* n_preallocs */
-	NULL            /* init */
-      };
-      
-      object_type = g_type_register_static (G_TYPE_OBJECT,
-                                            "PangoFontMap",
-                                            &object_info, 0);
-    }
-  
-  return object_type;
-}
-
+G_DEFINE_TYPE (PangoFontMap, pango_font_map, G_TYPE_OBJECT)
 
 static void
 pango_font_map_class_init (PangoFontMapClass *class)
@@ -65,6 +38,11 @@ pango_font_map_class_init (PangoFontMapClass *class)
   class->load_fontset = pango_font_map_real_load_fontset;
 }
 
+static void
+pango_font_map_init (PangoFontMap *fontmap)
+{
+}
+
 /**
  * pango_font_map_load_font:
  * @fontmap: a #PangoFontMap
@@ -81,6 +59,7 @@ pango_font_map_load_font  (PangoFontMap               *fontmap,
 			   const PangoFontDescription *desc)
 {
   g_return_val_if_fail (fontmap != NULL, NULL);
+  g_return_val_if_fail (pango_font_description_get_family (desc) != NULL, NULL);
 
   return PANGO_FONT_MAP_GET_CLASS (fontmap)->load_font (fontmap, context, desc);
 }
@@ -88,9 +67,9 @@ pango_font_map_load_font  (PangoFontMap               *fontmap,
 /**
  * pango_font_map_list_families:
  * @fontmap: a #PangoFontMap
- * @families: location to store a pointer to an array of PangoFontFamily *.
+ * @families: location to store a pointer to an array of #PangoFontFamily *.
  *            This array should be freed with g_free().
- * @n_families: location to store the number of elements in @descs
+ * @n_families: location to store the number of elements in @families
  * 
  * List all families for a fontmap. 
  **/
@@ -122,8 +101,9 @@ pango_font_map_load_fontset (PangoFontMap                 *fontmap,
 			     const PangoFontDescription   *desc,
 			     PangoLanguage                *language)
 {
-  g_return_val_if_fail (fontmap != NULL, 0);
-
+  g_return_val_if_fail (fontmap != NULL, NULL);
+  g_return_val_if_fail (pango_font_description_get_family (desc) != NULL, NULL);
+  
   return PANGO_FONT_MAP_GET_CLASS (fontmap)->load_fontset (fontmap, context, desc, language);
 }
 
@@ -132,7 +112,7 @@ pango_font_map_fontset_add_fonts (PangoFontMap          *fontmap,
 				  PangoContext          *context,
 				  PangoFontsetSimple    *fonts,
 				  PangoFontDescription  *desc,
-				  char                  *family)
+				  const char            *family)
 {
   char **aliases;
   int n_aliases;
@@ -172,6 +152,7 @@ pango_font_map_real_load_fontset (PangoFontMap               *fontmap,
   char **families;
   int i;
   PangoFontsetSimple *fonts;
+  static GHashTable *warned_fonts = NULL;
 
   families = g_strsplit (pango_font_description_get_family (desc), ",", -1);
 
@@ -198,11 +179,20 @@ pango_font_map_real_load_fontset (PangoFontMap               *fontmap,
       
       ctmp1 = pango_font_description_to_string (desc);
       pango_font_description_set_family_static (tmp_desc, "Sans");
-      ctmp2 = pango_font_description_to_string (tmp_desc);
-      
-      g_warning ("Couldn't load font \"%s\" falling back to \"%s\"", ctmp1, ctmp2);
+
+      if (!warned_fonts || !g_hash_table_lookup (warned_fonts, ctmp1))
+        {
+	  if (!warned_fonts)
+	    warned_fonts = g_hash_table_new (g_str_hash, g_str_equal);
+
+	  g_hash_table_insert (warned_fonts, g_strdup (ctmp1), GINT_TO_POINTER (1));
+
+	  ctmp2 = pango_font_description_to_string (tmp_desc);
+	  g_warning ("couldn't load font \"%s\", falling back to \"%s\", "
+		     "expect ugly output.", ctmp1, ctmp2);
+	  g_free (ctmp2);
+        }
       g_free (ctmp1);
-      g_free (ctmp2);
       
       pango_font_map_fontset_add_fonts (fontmap,
 					context,
@@ -223,11 +213,19 @@ pango_font_map_real_load_fontset (PangoFontMap               *fontmap,
       pango_font_description_set_weight (tmp_desc, PANGO_WEIGHT_NORMAL);
       pango_font_description_set_variant (tmp_desc, PANGO_VARIANT_NORMAL);
       pango_font_description_set_stretch (tmp_desc, PANGO_STRETCH_NORMAL);
-      ctmp2 = pango_font_description_to_string (tmp_desc);
+
+      if (!warned_fonts || !g_hash_table_lookup (warned_fonts, ctmp1))
+        {
+	  g_hash_table_insert (warned_fonts, g_strdup (ctmp1), GINT_TO_POINTER (1));
+
+	  ctmp2 = pango_font_description_to_string (tmp_desc);
       
-      g_warning ("Couldn't load font \"%s\" falling back to \"%s\"", ctmp1, ctmp2);
+	  g_warning ("couldn't load font \"%s\", falling back to \"%s\", "
+		     "expect ugly output.", ctmp1, ctmp2);
+	  g_free (ctmp2);
+	}
+
       g_free (ctmp1);
-      g_free (ctmp2);
       
       pango_font_map_fontset_add_fonts (fontmap,
 					context,
@@ -238,13 +236,34 @@ pango_font_map_real_load_fontset (PangoFontMap               *fontmap,
 
   pango_font_description_free (tmp_desc);
 
-  /* Everything failed, we are screwed, there is no way to continue
+  /* Everything failed, we are screwed, there is no way to continue,
+   * but lets just not crash here.
    */
   if (pango_fontset_simple_size (fonts) == 0)
-    {
-      g_warning ("All font failbacks failed!!!!");
-      exit (1);
-    }
+      g_warning ("All font fallbacks failed!!!!");
 
   return PANGO_FONTSET (fonts);
 }
+
+/**
+ * pango_font_map_get_shape_engine_type:
+ * @fontmap: a #PangoFontMap
+ * 
+ * Returns the render ID for shape engines for this fontmap.
+ * See the <structfield>render_type</structfield> field of
+ * #PangoEngineInfo.
+  * 
+ * Return value: the ID string for shape engines for
+ *  this fontmap. Owned by Pango, should not be modified
+ *  or freed.
+ *
+ * Since: 1.4
+ **/
+G_CONST_RETURN char *
+pango_font_map_get_shape_engine_type (PangoFontMap *fontmap)
+{
+  g_return_val_if_fail (PANGO_IS_FONT_MAP (fontmap), NULL);
+  
+  return PANGO_FONT_MAP_GET_CLASS (fontmap)->shape_engine_type;
+}
+
