@@ -49,7 +49,7 @@ static void
 set_color (PangoCairoRenderer *crenderer,
 	   PangoRenderPart     part)
 {
-  PangoColor *color = pango_renderer_get_color (PANGO_RENDERER (crenderer), part);
+  PangoColor *color = pango_renderer_get_color ((PangoRenderer *) (crenderer), part);
   
   if (color)
     cairo_set_source_rgb (crenderer->cr,
@@ -58,6 +58,48 @@ set_color (PangoCairoRenderer *crenderer,
 			  color->blue / 65535.);
 }
 
+/* modifies cairo_set_line_width() without doing cairo_save/restore() */
+static void
+_pango_cairo_renderer_draw_frame (PangoCairoRenderer *crenderer,
+				  double              x,
+				  double              y,
+				  double              width,
+				  double              height,
+				  double              line_width)
+{
+  if (crenderer->do_path)
+    {
+      cairo_t *cr = crenderer->cr;
+      double d2 = line_width * .5, d = line_width;
+
+      /* we draw an outer box in one winding direction and an inner one in the
+       * opposite direction.  This works for both cairo windings rules.
+       *
+       * what we really want is cairo_stroke_to_path().
+       */
+
+      /* outer */
+      cairo_move_to (cr, x - d2, y - d2);
+      cairo_rel_line_to (cr, width + d, 0);
+      cairo_rel_line_to (cr, 0, height + d);
+      cairo_rel_line_to (cr, - (width + d), 0);
+      cairo_close_path (cr);
+
+      /* inner */
+      cairo_move_to (cr, x + d2, y + d2);
+      cairo_rel_line_to (cr, 0, height - d);
+      cairo_rel_line_to (cr, width - d, 0);
+      cairo_rel_line_to (cr, 0, - (height - d));
+      cairo_close_path (cr);
+    }
+  else
+    {
+      cairo_rectangle (crenderer->cr, x, y, width, height);
+      cairo_set_line_width (crenderer->cr, line_width);
+      cairo_stroke (crenderer->cr);
+    }
+}
+
 static void
 _pango_cairo_renderer_draw_box_glyph (PangoCairoRenderer *crenderer,
 				      PangoGlyphInfo     *gi,
@@ -69,17 +111,13 @@ _pango_cairo_renderer_draw_box_glyph (PangoCairoRenderer *crenderer,
   cairo_save (crenderer->cr);
   cairo_get_current_point (crenderer->cr, &temp_x, &temp_y);
 
-  cairo_rectangle (crenderer->cr,
-		   cx + 1.5,
-		   cy + 1.5 - PANGO_UNKNOWN_GLYPH_HEIGHT,
-		   (double)gi->geometry.width / PANGO_SCALE - 3.0,
-		   PANGO_UNKNOWN_GLYPH_HEIGHT - 3.0);
 
-  if (!crenderer->do_path)
-    {
-      cairo_set_line_width (crenderer->cr, 1.0);
-      cairo_stroke (crenderer->cr);
-    }
+  _pango_cairo_renderer_draw_frame (crenderer,
+				    cx + 1.5,
+				    cy + 1.5 - PANGO_UNKNOWN_GLYPH_HEIGHT,
+				    (double)gi->geometry.width / PANGO_SCALE - 3.0,
+				    PANGO_UNKNOWN_GLYPH_HEIGHT - 3.0,
+				    1.0);
 
   cairo_move_to (crenderer->cr, temp_x, temp_y);
   cairo_restore (crenderer->cr);
@@ -106,7 +144,7 @@ _pango_cairo_renderer_draw_unknown_glyph (PangoCairoRenderer *crenderer,
 
   hbi = _pango_cairo_font_get_hex_box_info ((PangoCairoFont *)font);      
 
-  if (!hbi || !_pango_cairo_font_install (PANGO_CAIRO_FONT (hbi->font), crenderer->cr))
+  if (!hbi || !_pango_cairo_font_install ((PangoCairoFont *) (hbi->font), crenderer->cr))
     {
       _pango_cairo_renderer_draw_box_glyph (crenderer, gi, cx, cy);
       goto done;
@@ -118,17 +156,12 @@ _pango_cairo_renderer_draw_unknown_glyph (PangoCairoRenderer *crenderer,
   cols = (ch > 0xffff ? 6 : 4) / rows;
   g_snprintf (buf, sizeof(buf), (ch > 0xffff) ? "%06X" : "%04X", ch);
 
-  cairo_rectangle (crenderer->cr,
-		   cx + hbi->pad_x * 1.5,
-		   cy + hbi->box_descent - hbi->box_height + hbi->pad_y * 0.5,
-		   (double)gi->geometry.width / PANGO_SCALE - 3 * hbi->pad_x,
-		   (hbi->box_height - hbi->pad_y));
-
-  if (!crenderer->do_path)
-    {
-      cairo_set_line_width (crenderer->cr, hbi->line_width);
-      cairo_stroke (crenderer->cr);
-    }
+  _pango_cairo_renderer_draw_frame (crenderer,
+				    cx + hbi->pad_x * 1.5,
+				    cy + hbi->box_descent - hbi->box_height + hbi->pad_y * 0.5,
+				    (double)gi->geometry.width / PANGO_SCALE - 3 * hbi->pad_x,
+				    (hbi->box_height - hbi->pad_y),
+				    hbi->line_width);
 
   x0 = cx + hbi->pad_x * 3.0;
   y0 = cy + hbi->box_descent - hbi->pad_y * 2;
@@ -163,7 +196,7 @@ pango_cairo_renderer_draw_glyphs (PangoRenderer     *renderer,
 				  int                x,
 				  int                y)
 {
-  PangoCairoRenderer *crenderer = PANGO_CAIRO_RENDERER (renderer);
+  PangoCairoRenderer *crenderer = (PangoCairoRenderer *) (renderer);
 
   /* cairo_glyph_t is 24 bytes */
 #define MAX_STACK 40
@@ -177,7 +210,7 @@ pango_cairo_renderer_draw_glyphs (PangoRenderer     *renderer,
   if (!crenderer->do_path)
     set_color (crenderer, PANGO_RENDER_PART_FOREGROUND);
 
-  if (!_pango_cairo_font_install (PANGO_CAIRO_FONT (font), crenderer->cr))
+  if (!_pango_cairo_font_install ((PangoCairoFont *) (font), crenderer->cr))
     {
       for (i = 0; i < glyphs->num_glyphs; i++)
 	{
@@ -250,7 +283,7 @@ pango_cairo_renderer_draw_rectangle (PangoRenderer     *renderer,
 				     int                width,
 				     int                height)
 {
-  PangoCairoRenderer *crenderer = PANGO_CAIRO_RENDERER (renderer);
+  PangoCairoRenderer *crenderer = (PangoCairoRenderer *) (renderer);
 
   if (!crenderer->do_path)
     {
@@ -354,7 +387,7 @@ pango_cairo_renderer_draw_error_underline (PangoRenderer *renderer,
 					   int            width,
 					   int            height)
 {
-  PangoCairoRenderer *crenderer = PANGO_CAIRO_RENDERER (renderer);
+  PangoCairoRenderer *crenderer = (PangoCairoRenderer *) (renderer);
   cairo_t *cr = crenderer->cr;
 
   if (!crenderer->do_path)
@@ -398,7 +431,7 @@ static PangoCairoRenderer *cached_renderer = NULL;
 static GStaticMutex cached_renderer_mutex = G_STATIC_MUTEX_INIT;
 
 static PangoCairoRenderer *
-acquire_renderer (void)
+acquire_renderer (gboolean *free_renderer)
 {
   PangoCairoRenderer *renderer;
 
@@ -410,16 +443,21 @@ acquire_renderer (void)
         cached_renderer = g_object_new (PANGO_TYPE_CAIRO_RENDERER, NULL);
 
       renderer = cached_renderer;
+      *free_renderer = FALSE;
     }
   else
-    renderer = g_object_new (PANGO_TYPE_CAIRO_RENDERER, NULL);
+    {
+      renderer = g_object_new (PANGO_TYPE_CAIRO_RENDERER, NULL);
+      *free_renderer = TRUE;
+    }
 
   return renderer;
 }
 
-release_renderer (PangoCairoRenderer *renderer)
+static void
+release_renderer (PangoCairoRenderer *renderer, gboolean free_renderer)
 {
-  if (G_LIKELY (renderer == cached_renderer))
+  if (G_LIKELY (!free_renderer))
     {
       renderer->cr = NULL;
       renderer->do_path = FALSE;
@@ -442,7 +480,8 @@ _pango_cairo_do_glyph_string (cairo_t          *cr,
 			      PangoGlyphString *glyphs,
 			      gboolean          do_path)
 {
-  PangoCairoRenderer *crenderer = acquire_renderer ();
+  gboolean free_renderer;
+  PangoCairoRenderer *crenderer = acquire_renderer (&free_renderer);
   PangoRenderer *renderer = (PangoRenderer *) crenderer;
 
   crenderer->cr = cr;
@@ -470,7 +509,7 @@ _pango_cairo_do_glyph_string (cairo_t          *cr,
       pango_renderer_deactivate (renderer);
     }
   
-  release_renderer (crenderer);
+  release_renderer (crenderer, free_renderer);
 }
 
 static void
@@ -478,7 +517,8 @@ _pango_cairo_do_layout_line (cairo_t          *cr,
 			     PangoLayoutLine  *line,
 			     gboolean          do_path)
 {
-  PangoCairoRenderer *crenderer = acquire_renderer ();
+  gboolean free_renderer;
+  PangoCairoRenderer *crenderer = acquire_renderer (&free_renderer);
   PangoRenderer *renderer = (PangoRenderer *) crenderer;
 
   crenderer->cr = cr;
@@ -487,7 +527,7 @@ _pango_cairo_do_layout_line (cairo_t          *cr,
 
   pango_renderer_draw_layout_line (renderer, line, 0, 0);
   
-  release_renderer (crenderer);
+  release_renderer (crenderer, free_renderer);
 }
 
 static void
@@ -495,7 +535,8 @@ _pango_cairo_do_layout (cairo_t     *cr,
 			PangoLayout *layout,
 			gboolean     do_path)
 {
-  PangoCairoRenderer *crenderer = acquire_renderer ();
+  gboolean free_renderer;
+  PangoCairoRenderer *crenderer = acquire_renderer (&free_renderer);
   PangoRenderer *renderer = (PangoRenderer *) crenderer;
   
   crenderer->cr = cr;
@@ -504,7 +545,7 @@ _pango_cairo_do_layout (cairo_t     *cr,
 
   pango_renderer_draw_layout (renderer, layout, 0, 0);
   
-  release_renderer (crenderer);
+  release_renderer (crenderer, free_renderer);
 }
 
 static void 
