@@ -33,7 +33,7 @@
 
 #include "pango-engine-private.h"
 #include "pango-fontmap.h"
-#include "pango-utils.h"
+#include "pango-impl-utils.h"
 
 #undef PANGO_DISABLE_DEPRECATED
 
@@ -98,7 +98,7 @@ struct _PangoXSizeInfo
 };
 
 static const struct {
-  const gchar *text;
+  const gchar text[12];
   PangoWeight value;
 } weights_map[] = {
   { "light",     300 },
@@ -115,7 +115,7 @@ static const struct {
 };
 
 static const struct {
-  const gchar *text;
+  const gchar text[4];
   PangoStyle value;
 } styles_map[] = {
   { "r", PANGO_STYLE_NORMAL },
@@ -124,7 +124,7 @@ static const struct {
 };
 
 static const struct {
-  const gchar *text;
+  const gchar text[16];
   PangoStretch value;
 } stretches_map[] = {
   { "normal",        PANGO_STRETCH_NORMAL },
@@ -268,7 +268,7 @@ static GList *fontmaps = NULL;
  * not be freed. If the font map for a display is no longer needed, it can 
  * be released with pango_x_shutdown_display().
  *
- * Returns: a #PangoXFontMap for @display.
+ * Return value: a #PangoXFontMap for @display.
  **/
 PangoFontMap *
 pango_x_font_map_for_display (Display *display)
@@ -459,7 +459,7 @@ pango_x_font_map_load_font (PangoFontMap               *fontmap,
   name = g_ascii_strdown (pango_font_description_get_family (description), -1);
   size = pango_font_description_get_size (description);
 
-  if (size <= 0)
+  if (size < 0)
     return NULL;
 
   font_family = g_hash_table_lookup (xfontmap->families, name);
@@ -1291,7 +1291,6 @@ pango_x_make_matching_xlfd (PangoFontMap *fontmap, char *xlfd, const char *chars
       if (match_scaleable)
 	{
 	  char *prefix_end, *p;
-	  char *size_end;
 	  int n_dashes = 0;
 	  int target_size;
 	  char *prefix;
@@ -1315,8 +1314,6 @@ pango_x_make_matching_xlfd (PangoFontMap *fontmap, char *xlfd, const char *chars
 	      p++;
 	    }
 	  
-	  size_end = p - 1;
-
 	  target_size = (int)((double)size / xfontmap->resolution + 0.5);
 	  prefix = g_strndup (closest_match, prefix_end - closest_match);
 	  result  = g_strdup_printf ("%s--%d-*-*-*-*-*-%s", prefix, target_size, charset);
@@ -1339,7 +1336,7 @@ pango_x_make_matching_xlfd (PangoFontMap *fontmap, char *xlfd, const char *chars
  *
  * Obtains the font cache associated with the given font map.
  *
- * Returns: the #PangoXFontCache of @font_map.
+ * Return value: the #PangoXFontCache of @font_map.
  **/
 PangoXFontCache *
 pango_x_font_map_get_font_cache (PangoFontMap *font_map)
@@ -1429,7 +1426,7 @@ pango_x_fontmap_atom_from_name (PangoFontMap *fontmap,
 }
 
 
-const char *
+G_CONST_RETURN char *
 pango_x_fontmap_name_from_atom (PangoFontMap *fontmap, 
 				Atom          atom)
 {
@@ -1633,7 +1630,7 @@ pango_x_family_list_faces (PangoFontFamily  *family,
     }
 }
 
-const char *
+G_CONST_RETURN char *
 pango_x_family_get_name (PangoFontFamily  *family)
 {
   PangoXFamily *xfamily = PANGO_X_FAMILY (family);
